''' Base Resource and Mixin classes, not to be used directly '''

from collections import namedtuple

from pyramid.security import DENY_ALL
from sqlalchemy import asc, desc
from sqlalchemy.inspection import inspect


class BaseResource(object):
    ''' Base Resource for all other resources '''

    __parent__ = None
    __name__ = None

    request = None
    model = None
    nodes = dict()

    nav_highlight = None

    def __init__(self, name, parent, sql_model_instance=None):
        self.__name__ = name
        self.__parent__ = parent
        self.request = parent.request
        self.model = sql_model_instance

        # call to super().__init_() needed to set up PaginationMixin
        super().__init__()

    def __acl__(self):
        ''' Access controll list '''
        return [ DENY_ALL ]


    def __getitem__(self, key):
        ''' returns child resources '''
        klass = self.nodes.get(key, None)
        if klass:
            return klass(key, self)
        try:
            return super().__getitem__(key)
        except AttributeError:
            raise KeyError()


    @classmethod
    def from_sqla(cls, sql_model_instance, parent):
        ''' initializes a resource from an SQLalchemy object '''
        primary_keys = inspect(sql_model_instance).identity
        if primary_keys is None:
            raise ValueError('Cannot init resource for primary key: None')
        elif len(primary_keys) != 1:
            raise ValueError('Cannot init resource for composite primary key')
        primary_key = str(primary_keys[0])
        return cls(primary_key, parent, sql_model_instance)


class Pagination(object):
    ''' calculates pagination information

    Available instance attributes

        count:    total number of items
        items:    number of items displayed per page, aliased by items_per_page
        first:    first page number
        last:     last page number
        current:  current page number
        previous: previous page number
        next:     next page number
        window:   page window, e.g:
                  lets assume the current page is 10 and window size is 7
                  self.window = [7, 8, 9, 10, 11, 12, 13]
    '''

    default_items = 25
    default_window_size = 7

    def __init__(self, current, count, items=None, window_size=None):
        ''' calculates pagination information

        Parameters:
            current: current pages
            count: total number of items
            items: number of items displayed per pages
            window_size: size of pagination window
        '''
        # ensure values are integers
        current = self._ensure_int(current, 1)
        count = self._ensure_int(count, 0)
        items = self._ensure_int(items, self.default_items)
        window_size = self._ensure_int(window_size, self.default_window_size)

        # set the simples values that won't change
        self.count = count
        self.items = self.items_per_page = items

        # calculate number of pages
        pages = (count - 1) // items + 1
        self.first = 1
        self.last = max(self.first, pages)
        self.current = self._is_valid(current, default=self.first)
        self.previous = self._is_valid(self.current - 1)
        self.next = self._is_valid(self.current + 1)

        # window calculations
        # example: lets assume the current page is 10 and window size is 7
        # self.window = [7, 8, 9, 10, 11, 12, 13]
        half_window = window_size // 2
        start = self.current - half_window
        end = self.current + half_window
        calculated_window = range(start, end + 1)
        self.window = [p for p in calculated_window if self._is_valid(p)]

    def _is_valid(self, page, default=None):
        ''' checks if the given page is valid, returns default if not '''
        if self.count and self.first <= page <= self.last:
            return page
        return default

    def _ensure_int(self, value, default):
        ''' converts the value to integer, returns default if it fails '''
        try:
            return int(value)
        except Exception:
            return default


# named tuple for parameters used in sorting
SortParameter = namedtuple('SortParameter', 'text field direction func')


class PaginationResourceMixin(object):
    ''' mixin providing pagination information for simple sql models

    class attributes that must be defined in child classes:
        sql_model_class:         sqlalchemy model class
        child_resource_class:    resource representing a sqlalchemy model
        default_sorting:         string for default soring behaviour,
                                 e.g. 'created_on.desc'
        default_items_per_page:  default number of items displayed per page


    available instance attributes:
        pages:      pagination information, see Pagination class
        sorting:    sorting parameter, processed from request.GET
        filters:    filter parameters, processed from request.GET

    '''

    # class attributes that must be defined in child classes
    sql_model_class = None
    child_resource_class = None
    default_sorting = None
    default_items_per_page = 25

    # attributes set by processing request.GET
    pages = None
    sorting = None
    filters = {}

    # keys for request.GET processing
    query_key_current_page = 'p'
    query_key_items_per_page = 'n'
    query_key_sorting = 'o'

    # base sqlalchemy query object
    _base_query = None


    def __init__(self):
        ''' sets parameters from request.GET '''
        # first we need to remove non-filter parameters from GET
        params = dict(self.request.GET)
        page = params.pop(self.query_key_current_page, 1)
        items = params.pop(
            self.query_key_items_per_page,
            self.default_items_per_page
            )
        sort = params.pop(self.query_key_sorting, self.default_sorting)

        # we can now setup a base query with applied filters
        self._base_query = self.prepare_filtered_query(
            self.request.dbsession,
            params
            )

        # with this base query, the pagination can be calculated:
        count = self._base_query.count()
        self.pages = Pagination(page, count, items)

        # and we should check that we can sort results later
        self.sorting = self.parse_sort_parameters(sort)
        if self.sorting is None:
            msg = 'Error in default sorting {}'.format(self.default_sorting)
            raise ValueError(msg)


    def prepare_filtered_query(self, dbsession, filter_params):
        ''' setup the base filtered query

        An example:

        def prepare_filtered_query(self, dbsession, filter_params):
            query = dbsession.query(self.sql_model_class)

            by_username = filter_params.get('username', None)
            if by_username is not None:
                query = query.filter_by(username=by_username)
            # don't forget to remember the filter
            self.filters['username'] = by_username

            return query
        '''
        msg = 'Query setup must be implemented in child class'
        raise NotImplementedError(msg)


    def prepare_sorted_query(self, query, sorting):
        ''' add sorting to the base query

        An example:

        def prepare_sorted_query(self, query, sorting):
            model_field = getattr(self.sql_model_class, sorting.field)
            sort_func = sorting.func(model_field)
            return query.order_by(sort_func)
        '''
        msg = 'Query setup must be implemented in child class'
        raise NotImplementedError(msg)


    def parse_sort_parameters(self, sort_param):
        ''' parses a string that might contain sorting information '''
        sort_functions = { 'asc': asc, 'desc': desc}
        try:
            sort_param = sort_param.lower()
            field, direction = sort_param.split('.', 1)
            func = sort_functions[direction]
            return SortParameter(sort_param, field, direction, func)
        except (AttributeError, IndexError, KeyError, ValueError):
            return None


    def items(self):
        ''' returns the items of the current page as resources'''
        if not self.pages.count:
            return []

        # calculate the offset from the paging information
        offset = (self.pages.current - 1) * self.pages.items_per_page

        # prepare the query including sorting, offset and limit
        query = self.prepare_sorted_query(self._base_query, self.sorting)
        query = query.offset(offset).limit(self.pages.items_per_page)

        # return a list of resources representing the items found by the query
        return [
            self.child_resource_class.from_sqla(item, self)
            for item
            in query.all()
            ]


    def query_params(self, *args, **kwargs):
        ''' returns a dict with query parameters for a new request

        The entries are set by the parsed request.GET parameters used to
        construct the queries. If a new request comes in with the parameters
        provided, it will return the same child resources in the same order.
        the query parameters can be overridden by providing tuples or
        keyword arguments with new values.
        '''
        params = {
            self.query_key_current_page: self.pages.current,
            self.query_key_items_per_page: self.pages.items,
            self.query_key_sorting: self.sorting.text
            }
        params.update(self.filters)
        params.update(args)
        params.update(kwargs)
        # remove items that have None as a value
        filtered = {k: v for k, v in params.items() if v is not None}
        return filtered


    def url(self, *args, **kwargs):
        ''' shortcut for creating a url pointing to this resource

        shortcut for:
        request.resource_url(self, query=self.query_params(*args, **kwargs))
        '''
        params = self.query_params(*args, **kwargs)
        return self.request.resource_url(self, query=params)


    def __getitem__(self, key):
        ''' returns a child resource representing a sqlalchemy model '''
        model = self.request.dbsession.query(self.sql_model_class).get(key)
        if not model:
            raise KeyError()
        return self.child_resource_class.from_sqla(model, self)

