from sqlalchemy import or_

from pyramid.security import Allow, Authenticated, Deny, DENY_ALL, Everyone

from .base import BaseResource, PaginationResourceMixin
from ordr2.models import Category, Order, OrderStatus



class OrderResource(BaseResource):
    ''' Resource representing one order '''

    def __acl__(self):
        ''' Access controll list '''
        acl = [
            (Allow, 'role:user', 'view'),
            (Allow, 'role:user', 'create'),
            (Allow, 'role:purchaser', 'edit'),
            (Allow, 'role:purchaser', 'delete'),
            ]
        # open orders may be edited and deleted by the user that placed them
        if self.model.status == OrderStatus.OPEN:
            acl.append(
                (Allow, 'user:' + str(self.model.created_by), 'edit')
                )
            acl.append(
                (Allow, 'user:' + str(self.model.created_by), 'delete')
                )
        acl.append(DENY_ALL)
        return acl


class OrderList(BaseResource, PaginationResourceMixin):
    ''' Resource representing a list of orders '''

    sql_model_class = Order
    child_resource_class = OrderResource
    default_sorting = 'created.desc'
    default_items_per_page = 12

    def __acl__(self):
        ''' Access controll list '''
        return [
            (Allow, 'role:user', 'view'),
            (Allow, 'role:user', 'create'),
            (Allow, 'role:purchaser', 'edit'),
            (Allow, 'role:purchaser', 'delete'),
            DENY_ALL
            ]

    def prepare_filtered_query(self, dbsession, filter_params):
        ''' setup the base filtered query '''
        query = dbsession.query(self.sql_model_class)

        # filter by status
        status_name = filter_params.get('status', None)
        try:
            status_name = status_name.lower()
            status = OrderStatus(status_name)
            query = query.filter_by(status=status)
        except (AttributeError, ValueError):
            status_name = None
        self.filters['status'] = status_name

        # filter by user
        user_name = filter_params.get('user', None)
        if user_name:
            query = query.filter_by(created_by=user_name)
        self.filters['user'] = user_name

        # filter by search term
        search = filter_params.get('search', None)
        if search:
            term = '%{}%'.format(search)
            query = query.filter(
                or_(
                    self.sql_model_class.cas_description.ilike(term),
                    self.sql_model_class.vendor.ilike(term),
                    self.sql_model_class.catalog_nr.ilike(term),
                    self.sql_model_class.account.ilike(term),
                    self.sql_model_class.created_by.ilike(term)
                    )
                )
        self.filters['search'] = search

        return query

    def prepare_sorted_query(self, query, sorting):
        ''' add sorting to the base query '''
        available_fields = {
            'cas': 'cas_description',
            'category': 'category',
            'catalog': 'catalog_nr',
            'vendor': 'vendor',
            'price': 'unit_price',
            'amount': 'amount',
            'total': 'total_price',
            'created': 'created_date',
            'user': 'created_by',
            'status': 'status',
            }
        name = available_fields.get(sorting.field, None)
        model_field = getattr(self.sql_model_class, name, None)
        if model_field:
            sort_func = sorting.func(model_field)
            query = query.order_by(sort_func)

        # add default sorting
        default_sort = self.parse_sort_parameters(self.default_sorting)
        if sorting.field != default_sort.field:
            default_sort = self.parse_sort_parameters(self.default_sorting)
            query = self.prepare_sorted_query(query, default_sort)

        return query
