''' Account Registration and Settings '''

import deform

from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import render
from pyramid.security import remember, forget
from pyramid.view import view_config

from ordr2.events import UserLogIn
from ordr2.models import User, Role
from ordr2.schemas.account import (
    ResetPasswordSchema,
    RegistrationSchema,
    SettingsSchema
    )

# below this password length a warning is displayed
MIN_PW_LENGTH = 12

# user log in and log out

@view_config(
    context='ordr2:resources.Account',
    name='login',
    permission='login',
    request_method='GET',
    renderer='ordr2:templates/account/login.jinja2'
    )
def login_form(context, request):
    ''' display a login form '''
    return {}


@view_config(
    context='ordr2:resources.Account',
    name='login',
    permission='login',
    request_method='POST'
    )
def login(context, request):
    ''' loging in a user '''
    username = request.POST.get('username')
    password = request.POST.get('password')

    # Form validation is not done for login forms,
    # either the data represents a user or not.
    user = request.dbsession.query(User).filter_by(user_name=username).first()
    if user is not None:
        if user.is_active and user.check_password(password):
            headers = remember(request, user.id)
            event = UserLogIn(request, user)
            request.registry.notify(event)
            return HTTPFound(
                request.resource_path(request.root, 'orders'),
                headers=headers
                )

    request.flash(
        'error',
        'Oh snap! You entered the wrong unsername and/or password.',
        '''Please try it again. If you still can not log in make sure that
        your account is activated and you haven't enabled caps lock on
        your keyboard.''',
        dismissable=False
        )
    return HTTPFound(request.resource_path(context, 'login'))


@view_config(
    context='ordr2:resources.Account',
    name='logout',
    permission='logout'
    )
def logout(context, request):
    ''' logout of a user '''
    if request.user:
        pass
        # request.session.flash(MSG_LOGOUT, 'success')
    headers = forget(request)
    return HTTPFound(
        request.resource_path(request.root, 'about'),
        headers=headers
        )


# user registration

@view_config(
    context='ordr2:resources.Account',
    name='register',
    permission='register',
    request_method='GET',
    renderer='ordr2:templates/account/register.jinja2'
    )
def registration_form(context, request):
    ''' display a registration form '''
    context.nav_highlight = 'register'
    form = RegistrationSchema.as_form(request)
    return {'form': form}


@view_config(
    context='ordr2:resources.Account',
    name='register',
    permission='register',
    request_method='POST',
    renderer='ordr2:templates/account/register.jinja2'
    )
def registration_form_processing(context, request):
    ''' process a submitted registration form '''

    if 'Cancel' in request.POST:
        return HTTPFound(request.resource_path(request.root))

    form = RegistrationSchema.as_form(request)
    data = request.POST.items()
    try:
        appstruct = form.validate(data)
    except deform.ValidationFailure as e:
        context.nav_highlight = 'register'
        return {'form': form}

    # form validation successfull, create user
    account = User(
        user_name=appstruct['user_name'],
        first_name=appstruct['first_name'],
        last_name=appstruct['last_name'],
        email=appstruct['email'],
        role=Role.NEW
        )
    account.set_password(appstruct['password'])
    request.dbsession.add(account)

    request.flash(
        'success',
        'Your account <em>{}</em> has been created.'.format(account.user_name),
        dismissable=False
        )
    if len(appstruct['password']) < MIN_PW_LENGTH:
        request.flash(
            'warning',
            'You should really consider using a longer password.',
            dismissable=False
            )
    return HTTPFound(request.resource_path(context, 'registered'))


@view_config(
    context='ordr2:resources.Account',
    name='registered',
    permission='register',
    renderer='ordr2:templates/account/register_sucessful.jinja2'
    )
def registration_sucessful(context, request):
    ''' registration was sucessfull '''
    return {}


# user settings

@view_config(
    context='ordr2:resources.Account',
    name='settings',
    permission='settings',
    request_method='GET',
    renderer='ordr2:templates/account/settings.jinja2'
    )
def settings_form(context, request):
    ''' display the user settings form '''
    form = SettingsSchema.as_form(request)

    form_data = {
        'general': {
            'user_name': request.user.user_name,
            'first_name': request.user.first_name,
            'last_name': request.user.last_name,
            'email': request.user.email,
            'role': request.user.role.value.capitalize()
            }
        }
    form.set_appstruct(form_data)
    return {'form': form}


@view_config(
    context='ordr2:resources.Account',
    name='settings',
    permission='settings',
    request_method='POST',
    renderer='ordr2:templates/account/settings.jinja2'
    )
def settings_form_processing(context, request):
    ''' process the user settings form '''

    if 'Cancel' in request.POST:
        return HTTPFound(request.resource_url(request.root))

    form = SettingsSchema.as_form(request)
    data = request.POST.items()
    try:
        appstruct = form.validate(data)
    except deform.ValidationFailure as e:
        return {'form': form}

    # form validation sucessful, change settings
    request.user.first_name = appstruct['general']['first_name']
    request.user.last_name = appstruct['general']['last_name']
    request.user.email = appstruct['general']['email']
    if appstruct['change_password']['new_password']:
        request.user.set_password(appstruct['change_password']['new_password'])
        if len(appstruct['change_password']['new_password']) < MIN_PW_LENGTH:
            request.flash(
                'warning',
                'You should really consider using a longer password.'
                )

    request.flash('success', 'Your account information has been updated.')

    return {'form': form}


# passwort reset links

@view_config(
    context='ordr2:resources.PasswordResetAccount',
    permission='reset',
    request_method='GET',
    renderer='ordr2:templates/account/password_reset.jinja2'
    )
def reset_password_form(context, request):
    ''' display the password reset form '''
    form = ResetPasswordSchema.as_form(request)
    return {'form': form}


@view_config(
    context='ordr2:resources.PasswordResetAccount',
    permission='reset',
    request_method='POST',
    renderer='ordr2:templates/account/password_reset.jinja2'
    )
def reset_password_form_processing(context, request):
    ''' process the password reset form '''

    if 'Cancel' in request.POST:
        return HTTPFound(request.resource_url(request.root))

    form = ResetPasswordSchema.as_form(request)
    data = request.POST.items()
    try:
        appstruct = form.validate(data)
    except deform.ValidationFailure as e:
        return {'form': form}

    # form validation sucessful, change password and remove reset token
    context.model.set_password(appstruct['new_password'])
    context.model.password_reset = ''

    request.flash(
        'success',
        'Password reset successful',
        'Please Log In with your new password',
        )
    if len(appstruct['new_password']) < MIN_PW_LENGTH:
        request.flash(
            'warning',
            'You should really consider using a longer password.'
            )

    return HTTPFound(request.resource_url(request.root, 'account', 'login'))


